@@ -15,11 +15,11 @@ Passwd::Keyring::PWSafe3 - Password storage based on Password Safe encrypted fil
 
 =head1 VERSION
 
-Version 0.20
+Version 0.21
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 our $APP_NAME = "Passwd::Keyring";
 our $FOLDER_NAME = "Perl-Passwd-Keyring";
@@ -27,25 +27,16 @@ our $FOLDER_NAME = "Perl-Passwd-Keyring";
 =head1 SYNOPSIS
 
 Password Safe implementation of L<Passwd::Keyring>. Passwords are
-stored in the Password Safe (L<http://passwordsafe.sourceforge.net>)
+stored in the L<Password Safe|http://passwordsafe.sourceforge.net>
 encrypted file.
 
-This module does not require Password Safe to be installed, and can be
-used as generic "store many passwords in file encrypted with single
-master password" storage. Password Safe GUI, if installed, may help
-the user to review, modify, or delete saved passwords.
-
-Note: actual handling of Password Safe format is based on L<Crypt::PWSafe3>
-module. Passwd::Keyring::PWSafe3 just wraps it into the interface compatible
-with other Passwd::Keyring backends.
-
     use Passwd::Keyring::PWSafe3;
 
     my $keyring = Passwd::Keyring::PWSafe3->new(
          app=>"blahblah scraper",
          group=>"Johnny web scrapers",
-         file=>"/home/joe/secrets.pwsafe3",        # HOME / passwd-keyring.pwsafe3 by default
-         master_password=>"very secret password",  # Or callback. See ->new docs below
+         file=>"/home/joe/secrets.pwsafe3",        # ~/passwd-keyring.pwsafe3 by default
+         master_password=>"very secret password",  # Or callback. See below
     );
 
     my $username = "John";  # or get from .ini, or from .argv...
@@ -55,7 +46,7 @@ with other Passwd::Keyring backends.
         $password = <somehow interactively prompt for password>;
 
         # securely save password for future use
-        $keyring->set_password($username, "blahblah.com");
+        $keyring->set_password($username, $password, "blahblah.com");
     }
 
     login_somewhere_using($username, $password);
@@ -63,20 +54,42 @@ with other Passwd::Keyring backends.
         $keyring->clear_password($username, "blahblah.com");
     }
 
-Note: see L<Passwd::Keyring::Auto::KeyringAPI> for detailed comments
-on keyring method semantics (this document is installed with
+=head1 DESCRIPTION
+
+This module does not require Password Safe to be installed, and can be
+used as generic "store many passwords in file encrypted with single
+master password" storage. Password Safe GUI, if installed, may help
+the user to review, modify, or delete saved passwords.
+
+=over 4
+
+Official GUIs can be freely downloaded from L<the official
+site|http://passwordsafe.sourceforge.net> - both Windows and (beta)
+Linux versions are available. Apart from them there exist
+various L<compatible tools|http://passwordsafe.sourceforge.net/relatedprojects.shtml>,
+for example <Pasaffe for Gnome|https://launchpad.net/pasaffe>  or
+<PwSafe for Mac|http://passwordsafe.sourceforge.net/relatedprojects.shtml>.
+
+=back
+
+Actual handling of Password Safe format is based on L<Crypt::PWSafe3>
+module. Passwd::Keyring::PWSafe3 just wraps it into the interface
+compatible with other Passwd::Keyring backends.
+
+See L<Passwd::Keyring::Auto::KeyringAPI> for detailed comments
+on keyring methods (this document is installed with
 C<Passwd::Keyring::Auto> package).
 
 =head1 CAVEATS
 
 Underlying module (L<Crypt::PWSafe3>) in fact rewrites the whole file
-on every save (with the complete password list as read on init). This
-means that any attempts to use the file paralelly from a few programs,
-or from a few objects within one program, are doomed to cause lost
-updates. Also, all passwords from the file are kept in (unprotected)
-memory while keyring object is active. Therefore, it is recommended to
-use separate .psafe3 file for Passwd::Keyring::PWSafe3, not mixing it
-with possibly used normal Password Safe database, and to keep keyring
+on every save and keeps all passwords cached in memory while active.
+This means, that any attempts to use the file paralelly from a few
+programs, or from a few objects within one program, are doomed to
+cause lost updates. Also, all passwords from the file are kept in
+(unprotected) memory while keyring object is active. Therefore, it is
+recommended to use separate .psafe3 file for Passwd::Keyring::PWSafe3,
+not mixing it with normal Password Safe database, and to keep keyring
 object for a short time only, especially if modifications happen.
 
 There are some limitations in L<Crypt::PWSafe3> handling of Password
@@ -100,30 +113,54 @@ Username and password are ... well, used as username and password.
 Initializes the processing. Croaks if Crypt::PWSafe3 is not installed or
 master password is invalid. May create password file if it is missing.
 
-Handled named parameters: 
+Handled parameters:
 
-- app - symbolic application name (used in password notes)
+=over 4
 
-- group - name for the password group (used as folder name)
+=item app
 
-- file - location of .pwsafe3 file. If not given, C<passwd-keyring.pwsafe3> in user home directory is used. Will be created if does not exist. Note: absolute path is required, relative paths are very error prone.
+Symbolic application name (used in password notes)
 
-- master_password - password required to unlock the file. Can be
-  specified as string, or as callback returning a string (usually some
-  way of interactively asking user for the password).  The callback
-  gets two parameters: app and file.
+=item group
 
-  If this param is missing, module will prompt interactively for this
-  password using console prompt.
+Name for the password group (used as folder name)
 
-- lazy_save - if given, asks not to save the file after every change
-  (saving is fairly time consuming), but only when $keyring->save
-  is called or when keyring is destroyed.
+=item file
 
-Note: it of course does not make much sense to keep app passwords in encrypted
-storage if master password is saved in plain text. The module most natural
-usage is to interactively ask for master password (and use it to protect
-noticeable number of application-specific passwords).
+Location of .pwsafe3 file. If not given, C<passwd-keyring.pwsafe3> in
+user home directory is used. Will be created if does not exist. Note:
+absolute path is required, relative paths are very error prone.
+
+=item master_password 
+
+Password required to unlock the file. Can be given as string, or
+as callback returning a string (usually some way of interactively
+asking user for the password).  The callback gets two parameters: app
+and file.
+
+If this param is missing, module will prompt interactively for this
+password using console prompt.
+
+=item lazy_save
+
+if given, asks not to save the file after every change (saving is
+fairly time consuming), but only when $keyring->save is called or when
+keyring is destroyed.
+
+=back
+
+Note: it of course does not make much sense to keep app passwords in
+encrypted storage if master password is saved in plain text. The
+module most natural usage is to interactively ask for master password
+(and use it to protect noticeable number of application-specific
+passwords).
+
+Ideas of how to workaround this obstacle are welcome. I loosely
+consider either caching master password per desktop session
+(implementing sht. similar to ssh-agent/gpg-agent or using one of
+those somehow), or integrating the tool with PAM to use actual system
+password, or both - but while it seems doable on Linux, cross platform
+solution is not so easy.
 
 =cut
 
@@ -134,7 +171,7 @@ sub new {
     $self->{app} = $args{app} || 'Passwd::Keyring::PWSafe3';
     $self->{group} = $args{group} || 'Passwd::Keyring';
     $self->{lazy_save} = $args{lazy_save};
-    my $file = $args{file} || File::Spec->catfile($ENV{HOME}, "passwd-keyring.pwsafe3");
+    my $file = $args{file} || File::Spec->catfile(File::HomeDir->users_data, "passwd-keyring.pwsafe3");
 
     unless(File::Spec->file_name_is_absolute($file)) {
         croak("Absolute path to .pwsafe3 file is required, but relative path '$file' given");
@@ -184,7 +221,7 @@ sub _prompt_for_password {
     }
 }
 
-# Zwraca rekord dla danych parametrów. Jeśli go nie ma, zwraca undef
+# Find data record for given params, or undef if not found
 sub _find_record {
     my ($self, $username, $realm) = @_;
     my $group = $self->{group};
@@ -294,8 +331,6 @@ sub is_persistent {
 
 Marcin Kasperski
 
-Approach inspired by L<http://www.perlmonks.org/?node_id=869620>.
-
 =head1 BUGS
 
 Please report any bugs or feature requests to 
